@@ -1,11 +1,9 @@
 # requirement
 # original link
 # https://github.com/Isabek/XmlToTxt
-
 """
 declxml==0.9.1
 """
-
 """
 xml and txt are two formats of object detection labels, they are same labels just in different format
 
@@ -21,28 +19,38 @@ import declxml as xml
 
 class ObjectMapper(object):
     def __init__(self):
-        self.processor = xml.user_object("annotation", Annotation, [
-            xml.user_object("size", Size, [
-                xml.integer("width"),
-                xml.integer("height"),
-            ]),
-            xml.array(
-                xml.user_object("object", Object, [
-                    xml.string("name"),
-                    xml.user_object("bndbox", Box, [
-                        xml.floating_point("xmin"),
-                        xml.floating_point("ymin"),
-                        xml.floating_point("xmax"),
-                        xml.floating_point("ymax"),
-                    ], alias="box")
+        self.processor = xml.user_object(
+            "annotation", Annotation, [
+                xml.user_object("size", Size, [
+                    xml.integer("width"),
+                    xml.integer("height"),
                 ]),
-                alias="objects"
-            ),
-            xml.string("filename")
-        ])
+                xml.array(
+                    xml.user_object(
+                        "object", Object, [
+                            xml.string("name"),
+                            xml.user_object(
+                                "bndbox",
+                                Box, [
+                                    xml.floating_point("xmin"),
+                                    xml.floating_point("ymin"),
+                                    xml.floating_point("xmax"),
+                                    xml.floating_point("ymax"),
+                                ],
+                                alias="box"
+                            )
+                        ]
+                    ),
+                    alias="objects"
+                ),
+                xml.string("filename")
+            ]
+        )
 
     def bind(self, xml_file_path, xml_dir):
-        ann = xml.parse_from_file(self.processor, xml_file_path=os.path.join(xml_dir, xml_file_path))
+        ann = xml.parse_from_file(
+            self.processor, xml_file_path=os.path.join(xml_dir, xml_file_path)
+        )
         ann.filename = xml_file_path
         return ann
 
@@ -63,7 +71,9 @@ class Annotation(object):
         self.filename = None
 
     def __repr__(self):
-        return "Annotation(size={}, object={}, filename={})".format(self.size, self.objects, self.filename)
+        return "Annotation(size={}, object={}, filename={})".format(
+            self.size, self.objects, self.filename
+        )
 
 
 class Size(object):
@@ -92,7 +102,9 @@ class Box(object):
         self.ymax = None
 
     def __repr__(self):
-        return "Box(xmin={}, ymin={}, xmax={}, ymax={})".format(self.xmin, self.ymin, self.xmax, self.ymax)
+        return "Box(xmin={}, ymin={}, xmax={}, ymax={})".format(
+            self.xmin, self.ymin, self.xmax, self.ymax
+        )
 
 
 class Reader(object):
@@ -111,7 +123,8 @@ class Reader(object):
 
     @staticmethod
     def get_classes(filename):
-        with open(os.path.join(os.path.dirname(os.path.realpath('__file__')), filename), "r", encoding="utf8") as f:
+        with open(os.path.join(os.path.dirname(os.path.realpath('__file__')), filename), "r",
+                  encoding="utf8") as f:
             lines = f.readlines()
             return {value: key for (key, value) in enumerate(list(map(lambda x: x.strip(), lines)))}
 
@@ -135,11 +148,14 @@ class Transformer(object):
         xml_files = reader.get_xml_files()
         object_mapper = ObjectMapper()
         annotations = object_mapper.bind_files(xml_files, xml_dir=self.xml_dir)
-        self.write_to_txt(annotations)
+        if len(annotations) > 0:
+            self.write_to_txt(annotations)
 
     def write_to_txt(self, annotations):
         for annotation in annotations:
-            output_path = os.path.join(self.out_dir, self.darknet_filename_format(annotation.filename))
+            output_path = os.path.join(
+                self.out_dir, self.darknet_filename_format(annotation.filename)
+            )
             if not os.path.exists(os.path.dirname(output_path)):
                 os.makedirs(os.path.dirname(output_path))
             with open(output_path, "w+") as f:
@@ -149,9 +165,12 @@ class Transformer(object):
         result = []
         for obj in annotation.objects:
             if self.label_map_exist:
-                int_label = self.inv_map[obj.name]
+                try:
+                    int_label = self.inv_map[obj.name]
+                except:
+                    continue
             else:
-                isin =  obj.name in self.all_classes
+                isin = obj.name in self.all_classes
                 if not isin:
                     self.all_classes.append(obj.name)
                 int_label = self.all_classes.index(obj.name)
@@ -186,12 +205,20 @@ class Transformer(object):
         return "%s.txt" % pre
 
 
-def convert(xml_dir, out_dir, label_map=None):
+def convert(xml_dir, out_dir, label_map=None, keep_empty=True):
     """
 
     @param xml_dir:
     @param out_dir:
     @param label_map: dict of int label map to label name, if None, will generate random mapping
+    it has two purposes,
+    1. when it is give it will use the label map give by the dict, so apply this method couple times won't
+       same label won't be mapped to different int labels
+    2. when lap_map dict only contains some of the labels, say all labels have [person, dog, cat], but label_map is
+       {person: 0, cat: 1}, and the data created will omit dog annotations, on other word, dog will be treated as
+       background
+
+    @keep_empty: we can remove .txt files if there is no label in it, but we do suggest have some images without labels
     @return:
     """
     if not os.path.exists(xml_dir):
@@ -208,6 +235,21 @@ def convert(xml_dir, out_dir, label_map=None):
     transformer = Transformer(xml_dir=xml_dir, out_dir=out_dir, label_map=label_map)
     transformer.transform()
 
+    if not keep_empty:
+        label_path = []
+        for root, dirs, files in os.walk(out_dir):
+            for file in files:
+                if file.endswith(".txt"):
+                    the_path = os.path.join(root, file)
+                    label_path.append(the_path)
+        for lb_p in label_path:
+            with open(lb_p) as f:
+                lines = f.read().splitlines()
+            if len(lines) >0:
+                continue
+            else:
+                os.remove(lb_p)
+
     return transformer.label_map
 
 
